from Screens.Screen import Screen
from Components.Label import Label
from Components.ActionMap import ActionMap
from Screens.MessageBox import MessageBox
from Plugins.Plugin import PluginDescriptor

from os import sysconf as os_sysconf
import threading
import Queue
import time, random

CPU_CORES = os_sysconf('SC_NPROCESSORS_ONLN')
print "[plugin] CPU cores:",CPU_CORES

###########################################################################

class MPWorker(threading.Thread):
	"""MPWorker: Multitask Test Class"""
	def __init__(self, queue, cpu=0):
		self.__cpu = cpu
		self.__queue = queue
		threading.Thread.__init__(self)

	def run(self):
		while 1:
			task = self.__queue.get()
			if task is None:
				break # reached end of queue
			else:
				print "[%s]"%self.__class__.__name__,"task", task[1], "running cpu core:",self.__cpu
				time.sleep(random.randint(10, 100) / 100.0)
				task[0](task[1])
				print "[%s]"%self.__class__.__name__,"task", task[1], "finished!"

###########################################################################

class MyScreen(Screen):
	"""MyScreen: Test Screen Class"""
	skin = """
		<screen position="center,center" size="640,480" title="MyScreen" >
			<widget name="Label" position="10,60" size="400,120" font="Regular;20"/>
		</screen>"""

	def __init__(self, session, args = 0):
		self.session = session
		
		self.__queue = Queue.Queue(CPU_CORES)
		for i in range(CPU_CORES):
			MPWorker(self.__queue,i).start() # start a worker

		Screen.__init__(self, session)

		self["Label"] = Label(_("press OK for MP Test"))
		self["actions"] = ActionMap(["SetupActions"],
		{
			"ok": self.KeyOk,
			"cancel": self.KeyExit
		}, -1)
		
	def Hallo(self, msg=None):
		print "[%s]"%self.__class__.__name__, "Hallo Welt!", msg
		
	def KeyOk(self):
		for i in range(10):
			print "[%s]"%self.__class__.__name__, "push", i
			self.__queue.put((self.Hallo, str(i)))
		
	def KeyExit(self):
		for i in range(CPU_CORES):
			self.__queue.put(None)
		self.close(None)

###########################################################################

def main(session, **kwargs):	
	session.open(MyScreen)

###########################################################################

def Plugins(**kwargs):
	return PluginDescriptor(
			name="01 MyScreen",
			description="Multitask test Screen",
			where = PluginDescriptor.WHERE_PLUGINMENU,
			icon=None,
			fnc=main)
